#!/usr/bin/perl

use strict;
use IO::File;
use File::Path;
use File::stat;
use Fcntl qw(:flock);
use POSIX;
use Switch;
#use Archive::Zip;

my $DIR = "/usr/local/bin/ngn";
my ($yyyy, $mm, $dd);
my ($fileNumber, $stationNumber);
my $OUT = "/usr/local/bin/ngn/out";
my $IN = $DIR."/in";

my $help = "usage: convert.pl [key] [param]
 -i input directory, default $IN
 -o output directory, default $OUT";

#разбор входных параметров
my $var="1";
foreach my $arg (@ARGV)
{
  if( $var eq "-i")
  {
	$IN = $arg;
	print "IN = $IN \n";
	$var = "1";
  }
  elsif($var eq "-o")
  {
	$OUT = $arg;
	print "OUT = $OUT \n";
	$var = "1";
  }
  elsif($arg =~ /-i|-o/)
  {
	$var = $arg;
  }
}

if($ARGV[0] eq "--help")
{
  print "$help\n";
  exit(0);
}
# end
#обработка
opendir (DIR,"$IN");
print "start converting files from $IN \n";
foreach my $cur_file (readdir(DIR))
{
  if( $cur_file =~ /^Dag(\d{2})(\d{2})(\d{2})(\d{8}).dat$/ )  
  {
	  print "converting file $cur_file \n";
	  #($yyyy, $mm, $dd, $fileNumber) = ($1, $2, $3, $4, $5);
	  #получаем время последнего изменения файла. Игнорим дату из переменных, так как там дата 20, а изменен файл 21 в 00:00
	  my $st = stat("$IN/".$cur_file) or die "No $cur_file $!";
	  my $mt = $st->mtime;
	  my ( $sec, $min, $hour, $mday, $mon, $year, $wday ) = localtime($mt);
	  $year+=1900;
	  $mon+=1;
	  $mon = sprintf "%02d",$mon;
	  $hour = sprintf "%02d",$hour;
	  $mday = sprintf "%02d",$mday;
	  print "Время последнего изменения файла: $mday-$mon-$year , $hour:$min\n";
	  mkpath("$OUT/$year/$mon/" );
	  #my $out_file = $cur_file; # старое имя
	  my $out_file = $mday.'_'.$hour;
	  my $archName = $out_file;
	  $archName .= '.zip';
	  $out_file .= '.txt';
	  print "Новое имя файла $out_file \n";
	  open FILE_OUT, ">$OUT/$year/$mon/$out_file" || die "Can't open $out_file on write!\n";
	  parseFile( $cur_file );
	  #архивируем файл
	  my $response = `zip -j -9 $OUT/$year/$mon/$archName $OUT/$year/$mon/$out_file`;
	  #удаляем файл
	  $response = `rm $OUT/$year/$mon/$out_file`;
	  close FILE_OUT;	
  }
}
closedir (DIR);
#end

my %DATA = ();
sub parseFile()
{
	my $logFile = shift;
	open( FILE, "<:bytes:raw", "$IN/$logFile" ) or die "Cannot open $logFile : $!";
	my $data;
	my $count = 0;
	
	while( read( FILE, $data, 8 ) ) 
	{
		my @f = unpack( 'LSCC', $data );
		
		my $csn = $f[0];
		my $length = $f[1];
		my $netType = $f[2];
		my $billType = $f[3];
	
		if( $billType eq 1 )
		{
			fixedDetailBill( $data, $csn, $length, $netType, $billType );    
		}		
		else
		{
			read( FILE, $data, $length-8 );
		}
		$count ++;
	}

	close( FILE );
	print "INFO Count rows in file: $count";
}

sub fixedDetailBill
{
	my $data = shift;
	my $csn = shift;
	my $length = shift;
	my $netType = shift;
	my $billType = shift;
	
	read( FILE, $data, 3 );
	my @f = unpack( 'CCC', $data );

	read( FILE, $data, 6 );
	@f = unpack( 'CCCCCC', $data );
	
	my %beginDateTime = ( 'undef', 1 );
	my %endDateTime = ( 'undef', 1 );

	my $time = \%beginDateTime;
	
	my $month = $f[1];
	$month =~ s/^(\d)$/0$1/;
	my $dd = $f[2];
	$dd =~ s/^(\d)$/0$1/;
	my $hh = $f[3];
	$hh =~ s/^(\d)$/0$1/;
	my $mm = $f[4];
	$mm =~ s/^(\d)$/0$1/;
	my $ss = $f[5];
	$ss =~ s/^(\d)$/0$1/;
	$time->{ 'undef' } = 0;
	$time->{ 'year' } = $f[0] + 2000;
	$time->{ 'month' } = $month;
	$time->{ 'day' } = $dd;
	$time->{ 'hour' } = $hh;
	$time->{ 'minute' } = $mm;
	$time->{ 'second' } = $ss;

	read( FILE, $data, 6 );
	@f = unpack( 'CCCCCC', $data );
	
	my $time = \%endDateTime;
	
	my $month = $f[1];
	$month =~ s/^(\d)$/0$1/;
	my $dd = $f[2];
	$dd =~ s/^(\d)$/0$1/;
	my $hh = $f[3];
	$hh =~ s/^(\d)$/0$1/;
	my $mm = $f[4];
	$mm =~ s/^(\d)$/0$1/;
	my $ss = $f[5];
	$ss =~ s/^(\d)$/0$1/;
	$time->{ 'undef' } = 0;
	$time->{ 'year' } = $f[0] + 2000;
	$time->{ 'month' } = $month;
	$time->{ 'day' } = $dd;
	$time->{ 'hour' } = $hh;
	$time->{ 'minute' } = $mm;
	$time->{ 'second' } = $ss;

	read( FILE, $data, 4 );
	@f = unpack( 'L', $data );
	
	my $conversationTime = ceil($f[0]/100);

	read( FILE, $data, 50 );
	@f = unpack( 'SCH[32]SCH[32]SH[10]H[10]', $data );

	my $callerNumber = $f[2];
	$callerNumber =~ s/^(\d*)[f]*$/$1/;

	my $calledNumber = $f[5];
	$calledNumber =~ s/^(\d*)[f]*$/$1/;
	
	#trunk_group_in
	read( FILE, $data, 10 );
	@f = unpack( 'SSCCCCCC', $data );

	my $trunkGroupIn = $f[0];
	my $trunkGroupOut = $f[1];

	#termination code
	read( FILE, $data, 16 );
	@f = unpack( 'CSSSSLSC', $data );

	#connected_number
	read( FILE, $data, 58 );
	@f = unpack( 'H[32]SCH[32]CH[32]CCL', $data );

	my $connectedNumber = $f[0];
	$connectedNumber =~ s/^(\d*)[f]*$/$1/;
	my $chargeNumber = $f[3];
	$chargeNumber =~ s/^(\d*)[f]*$/$1/;
	my $dialNumber = $f[5];
	$dialNumber =~ s/^(\d*)[f]*$/$1/;

	#spared
	read( FILE, $data, 39 );

	my $filePos = tell( FILE );
	seek( FILE, $filePos + $length-200, 0 );


	my $yy = $beginDateTime{'year'};
	my $mm = $beginDateTime{'month'};
	my $dd = $beginDateTime{'day'};
	my $hh = $beginDateTime{'hour'};

	my $key = $yy."_".$mm."_".$dd."_".$hh;
	my $value = "$beginDateTime{'day'}.$beginDateTime{'month'}.$beginDateTime{'year'} $beginDateTime{'hour'}:$beginDateTime{'minute'}:$beginDateTime{'second'}\t$conversationTime\t$callerNumber\t".numA2e164($callerNumber)."\t$connectedNumber\t".numB2e164($connectedNumber)."\t$trunkGroupIn\t$trunkGroupOut\t0\n";

	$DATA{$key} .= $value;
	
	print FILE_OUT "$value\n"; 
}

sub numA2e164
{
	my $num = shift;
	chomp($num);

	#раз после % может быть что угодно, то поэтому .+ 
	if ($num =~ s/^87220000$/77220000/){;} #'87220000 :p
	elsif ($num =~ s/^8722(\d{6})$/7722$1/){;} #8722______ :p
	elsif ($num =~ s/^87228722(\d{6})$/7722$1/){;} #87228722______ substr(:p,5)
	elsif ($num =~ s/^299(\d{4})$/99$1/){;} # 299____ substr(:p,2)
	elsif ($num =~ s/7722(\d{6})^$/7722$1/){;} #7722______ '8722'||substr(:p,5)
	elsif ($num =~ s/^(\8722722d{6})$/$1/){;} #8722722______ substr(:p,8)
	elsif ($num =~ s/^722(\d{6})$/2722$1/){;} #722______ '8722'||substr(:p,4)
	elsif ($num =~ s/^822(\d{7})$/22$1/){;} #822_______ substr(:p,2)
	elsif ($num =~ s/^872(\d{7})$/2$1/){;} #872_______ substr(:p,3)
	elsif ($num =~ s/^8872(\d+)$/2$1/){;} #8872% substr(:p,4)
	elsif ($num =~ s/^(\d{6})$/7722$1/){;} #______ '8722'||:p
	elsif ($num =~ s/^8722(\d{6})(\d{2})$/7722/){;} #8722________ substr(:p,1,10)
	elsif ($num =~ s/^8722(\d{2})(\d{6})$/$1/){;} #8722________ substr(:p,5,6)
	elsif ($num =~ s/^872251(\d{6})$/51/){;} #872251______ substr(:p,5,6)
	elsif ($num =~ s/^78722(\d{6})$/7722$1/){;} #78722______ substr(:p,2)
	elsif ($num =~ s/^78722(\d{1})(\d{5})$/$1/){;} #78722______ substr(:p,6,6)
	elsif ($num =~ s/^810(\.+)$/$1/){;} #810% substr(:p,4)
	elsif ($num =~ s/^8559(\.+)$/9$1/){;} #8559% substr(:p,4)
	elsif ($num =~ s/^855(\.+)$/$1/){;} #855% substr(:p,4)
	elsif ($num =~ s/^853(\.+)$/$1/){;} #853% substr(:p,4)
	elsif ($num =~ s/^852(\.+)$/$1/){;} #852% substr(:p,4)
	elsif ($num =~ s/^858(\.+)$/$1/){;} #858% substr(:p,4)
	elsif ($num =~ s/^857(\.+)$/$1/){;} #857% substr(:p,4)
	elsif ($num =~ s/^(d{10})$/$1/){;} #__________ :p
	elsif ($num =~ s/^8(.+)$/$1/){;} #'8% substr(:p,2)
		
	return $num;
}

sub numB2e164
{
	my $num = shift;
	chomp($num);

	if ($num =~ s/^8722(\d{6})$/7722$1/){;} #8722______ :p
	elsif ($num =~ s/7722(\d{6})^$/7722$1/){;} #7722______ '8722'||substr(:p,5)
	elsif ($num =~ s/^8840(\.+)$/7108400$1/){;} #'8840% '810840'||substr(:p,4)
	elsif ($num =~ s/^722(\d{6})$/2722$1/){;} #722______ '8722'||substr(:p,4)
	elsif ($num =~ s/^822(\d{7})$/22$1/){;} #822_______ substr(:p,2)
	elsif ($num =~ s/^822(\d{6})$/22$1/){;} #822_______ substr(:p,2)
	elsif ($num =~ s/^82(\d{8})$/2$1/){;} #82________ substr(:p,2)
	elsif ($num =~ s/^872(\d{7})$/2$1/){;} # 872_______ substr(:p,3)
	elsif ($num =~ s/^872(\d{6})$/2$1/){;} #872______ substr(:p,3)
	elsif ($num =~ s/^887227(\d{5})$/7$1/){;} #887227_____ substr(:p,6)
	elsif ($num =~ s/^887229(\d{5})(.+)$/9$1$2/){;} #887229_____% substr(:p,6)
	elsif ($num =~ s/^88722(.+)$/2$1/){;} #'88722% substr(:p,5)
	elsif ($num =~ s/^8872(.+)$/2$1/){;} #'8872% substr(:p,4)
	elsif ($num =~ s/(\d{6})^$/7722$1/){;} #______ '8722'||:p
	elsif ($num =~ s/^(\d{2})$/$1/){;} #__ :p
	elsif ($num =~ s/^8940(.+)$/940$1/){;} #'8940% substr(:p,2)
	elsif ($num =~ s/^810(\.+)$/$1/){;} #810% substr(:p,4)
	elsif ($num =~ s/^8559(\.+)$/9$1/){;} #8559% substr(:p,4)
	elsif ($num =~ s/^855(\.+)$/$1/){;} #855% substr(:p,4)
	elsif ($num =~ s/^853(\.+)$/$1/){;} #853% substr(:p,4)
	elsif ($num =~ s/^852(\.+)$/$1/){;} #852% substr(:p,4)
	elsif ($num =~ s/^858(\.+)$/$1/){;} #858% substr(:p,4)
	elsif ($num =~ s/^857(\.+)$/$1/){;} #857% substr(:p,4)
	elsif ($num =~ s/^82(\d{7})$/2$1/){;} #82_______ substr(:p,2)
	elsif ($num =~ s/^8(.+)$/$1/){;} #'8% substr(:p,2)

	return $num;
}